#import <appkit/appkit.h>

typedef enum { C_L, C_S, C_D, C_GO, C_IF, C_TH, C_EL,
               C_E, C_A, C_PLUS, C_MINUS, C_MUL, C_DIV, C_APP,
               C_P, C_F, C_I,
               C_NOP, C_END, C_SAY, C_PLAY, C_ALERT } commandCode;
 
typedef struct _com {
  struct _com * succ;
  int           startPosition;
  int           endPosition;
  char          label[80];
  commandCode   code;
  char          argument[256];
} command;

@interface Controller: Object
{
  id programText;
  id constantsText;
  id variablesText;
  id inputText;
  id outputText;
  id accumulator;
  id messageText;
  id testResult;

  id questionPanel;
  id questionField;
  id answerField;

  id translation;
  id sayPathField;
  id sayAvailableButton;
  id soundPathField;
  id sndplayPathField;

  command * commands;
  id  variables;
  id  constants;
  command * nextCommand;
  int nextInputLine;
  char * directory;
  char * filename;
  BOOL   cancelRequested;
}

- init;
- awakeFromNib;
- defaultsOK:sender;
- revertDefaults:sender;
- setKey:(const char *)key toValue:(const char *)value inStringTable:table;
- setNextCommand:(command *)command;
- analyseConstantLine:(const char *)line;
- addCommand:(commandCode)code label:(char *)label argument:(char *)arg
  start:(int)start end:(int)end;
- missingArgument:(char *)arg inCommand:(char *)command;
- analyseProgramLine:(const char *)line start:(int)start end:(int)end;
- showVariables;
- (command *)searchCommandWithLabel:(char *)label;
- (const char *)getDirectValue:(const char *)name;
- (const char *)getVarOrConstOrDirectValue:(const char *)name;
- setVar:(const char *)name toValue:(const char *)value;
- addVariable:(const char *)name;
- addVariableOrConstOrDirectValue:(const char *)name;
- (const char*)input;
- output:(const char *)line;
- (const char *)askForValue:(const char *)question;
- answerOK:sender;
- answerCancel:sender;
- say:(const char *)value;
- (BOOL)fileExists:(const char *)name;
- (const char *)findSoundFile:(const char *)name;
- play:(const char *)name;
- alert:(const char *)value;
- performNextCommand;
- reset:sender;
- step:sender;
- run:sender;
- stop:sender;
- toggleProgramVisibility:sender;
- setMessage:(const char *)msg;
- save:sender;
- restore:sender;

@end
